// ntauth.cpp: implementation of the NTAuth class.
//             This might as well just be a function
//////////////////////////////////////////////////////////////////////
#include "ntauth.h"
#include "vncserver.h"

NTAuth::NTAuth(const BOOL adminsOnly) {
	AllowAdminsOnly = adminsOnly;
}

NTAuth::~NTAuth() {
// Nothing to clean up
}

// Returns 0 on success, non-zero on some failure
long NTAuth::authUser(const NTUserInfo* userInfo) {

	LPBYTE ui; 
	DWORD level;
	NET_API_STATUS rc;

	if(!userInfo)
		return -1;

	// Now make sure we're not trying to login under
	// a guest account (allow only Administrators and Users
	// on this local computer).  The admin is allowed to set 
	// whether only admins are allowed to authenticate, as well.
	level = 11;

	rc = NetUserGetInfo(NULL, userInfo->username, level, &ui);
	if(rc != NERR_Success) {
		NetApiBufferFree(&ui);
		return (long)rc;
	}

	if(((_USER_INFO_11*)ui)->usri11_priv != USER_PRIV_ADMIN  ) 
		if(AllowAdminsOnly || 
		  ((_USER_INFO_11*)ui)->usri11_priv != USER_PRIV_USER)  {
			NetApiBufferFree(ui);
			return -2;
		}

	NetApiBufferFree(ui);

	// Now check if the account is disabled or locked out
	level = 20;
	rc = NetUserGetInfo(NULL, userInfo->username, level, &ui);
	if(((_USER_INFO_20*)ui)->usri20_flags == UF_ACCOUNTDISABLE ||
	   ((_USER_INFO_20*)ui)->usri20_flags == UF_LOCKOUT )  {
		NetApiBufferFree(ui);
		return -3;
	}

	NetApiBufferFree(ui);

	// Okay, we're authenticating a user or a guest, now lets see if 
	// the password matches. If no domain is specified, use the local
	// domain.
	if(wcslen(userInfo->domain) > 0)
		rc = NetUserChangePassword(userInfo->domain, userInfo->username,
			userInfo->password, userInfo->password);
	else 
		rc = NetUserChangePassword(NULL,
								   userInfo->username,
								   userInfo->password, 
								   userInfo->password);

	// Just because we got a non-zero return code here doesn't mean there is an error.
	// Some systems will report a too-short password even if the password is 
	// valid.  Don't know why.  Anyway, here are some errors which could occur
	// when we try to change the password (to itself), but these particular errors
	// should be ignored.  
	switch(rc) {
	case NERR_PasswordCantChange:
	case NERR_PasswordHistConflict:
	case NERR_PasswordTooShort:
	case NERR_PasswordTooRecent:
	case NERR_DatabaseUpToDate:
		rc = 0;
		break;
	default:
		break;
	};
 
	// This returns 0 on NO_ERROR, i.e. the user specified the correct
	// username, password and domain and was able to authenticate.
	return rc;
}

